@@ -9,6 +9,7 @@ See the file COPYING for details.
 #include "batch_job.h"
 #include "hash_table.h"
 #include "copy_stream.h"
+#include "copy_tree.h"
 #include "debug.h"
 #include "domain_name_cache.h"
 #include "envtools.h"
@@ -88,7 +89,7 @@ int master_port = 0;
 int using_catalog = 0;
 
 static char *extra_worker_args=0;
-static const char *resource_args=0;
+static char *resource_args=0;
 static int abort_flag = 0;
 static const char *scratch_dir = 0;
 static const char *password_file = 0;
@@ -263,7 +264,7 @@ Count up the workers needed in a given list of masters, IGNORING how many
 workers are actually connected.
 */
 
-static int count_workers_needed( struct list *masters_list, int only_waiting )
+static int count_workers_needed( struct list *masters_list, int only_not_running )
 {
 	int needed_workers=0;
 	int masters=0;
@@ -285,14 +286,13 @@ static int count_workers_needed( struct list *masters_list, int only_waiting )
 		const int tl =       jx_lookup_integer(j,"tasks_left");
 
 		int capacity = master_workers_capacity(j);
-		int tasks = tr+tw+tl;
 
 		// first assume one task per worker
 		int need;
-		if(only_waiting) {
-			need = tw;
+		if(only_not_running) {
+			need = tw + tl;
 		} else {
-			need = tasks;
+			need = tw + tl + tr;
 		}
 
 		// enforce many tasks per worker
@@ -308,7 +308,7 @@ static int count_workers_needed( struct list *masters_list, int only_waiting )
 			need = MIN(need, capacity);
 		}
 
-		debug(D_WQ,"%s %s:%d %s tasks: %d capacity: %d workers needed: %d tasks running: %d",project,host,port,owner,tasks,capacity,need,tr);
+		debug(D_WQ,"%s %s:%d %s tasks: %d capacity: %d workers needed: %d tasks running: %d",project,host,port,owner,tw+tl+tr,capacity,need,tr);
 		needed_workers += need;
 		masters++;
 	}
@@ -337,14 +337,22 @@ static void set_worker_resources_options( struct batch_queue *queue )
 		}
 	}
 
+	free(resource_args);
 	resource_args = xxstrdup(buffer_tostring(&b));
+
 	buffer_free(&b);
 }
 
 static int submit_worker( struct batch_queue *queue )
 {
 	char *cmd;
-	const char *worker = runos_os ? "work_queue_worker" : "./work_queue_worker";
+	char *worker;
+
+	if(runos_os || k8s_worker_image) {
+		worker = xxstrdup("work_queue_worker");
+	} else {
+		worker = string_format("./%s", worker_command);
+	}
 
 	if(using_catalog) {
 		cmd = string_format(
@@ -359,9 +367,6 @@ static int submit_worker( struct batch_queue *queue )
 		);
 	}
 	else {
-		if(k8s_worker_image) {
-			worker = "work_queue_worker";
-		} 
 		cmd = string_format(
 		"%s %s %d -t %d -C '%s' -d all -o worker.log %s %s %s",
 		worker,
@@ -382,8 +387,8 @@ static int submit_worker( struct batch_queue *queue )
 		free(cmd);
 		cmd = newcmd;
 	}
-	
-	char *files = NULL;	
+
+	char *files = NULL;
 	if(!k8s_worker_image) {
 		files = string_format("work_queue_worker");
 	} else {
@@ -401,7 +406,7 @@ static int submit_worker( struct batch_queue *queue )
 		free(files);
 		files = newfiles;
 	}
-	
+
 	if(runos_os){
 		char* vc3_cmd = string_format("./vc3-builder --require cctools-statics -- %s",cmd);
 		char* temp = string_format("python %s %s %s",CCTOOLS_RUNOS_PATH,runos_os,vc3_cmd);
@@ -419,7 +424,14 @@ static int submit_worker( struct batch_queue *queue )
 
 	debug(D_WQ,"submitting worker: %s",cmd);
 
-	return batch_job_submit(queue,cmd,files,"output.log",batch_env,resources);
+
+	int status = batch_job_submit(queue,cmd,files,"output.log",batch_env,resources);
+
+	free(cmd);
+	free(files);
+	free(worker);
+
+	return status;
 }
 
 static void update_blacklisted_workers( struct batch_queue *queue, struct list *masters_list ) {
@@ -846,25 +858,32 @@ static void mainloop( struct batch_queue *queue )
 		}
 	
 		debug(D_WQ,"evaluating master list...");
-		int workers_needed    = count_workers_needed(masters_list, 0);
 		int workers_connected = count_workers_connected(masters_list);
-
-		debug(D_WQ,"%d total workers needed across %d masters",
-				workers_needed,
-				masters_list ? list_size(masters_list) : 0);
+		int workers_needed = 0;
 
 		if(foremen_regex)
 		{
+			/* If there are foremen, we only look at tasks not running in the
+			 * masters' list. The rest of the tasks will be counted as waiting
+			 * or running on the foremen. */
+			workers_needed = count_workers_needed(masters_list, /* do not count running tasks */ 1);
 			debug(D_WQ,"evaluating foremen list...");
 			foremen_list    = work_queue_catalog_query(catalog_host,-1,foremen_regex);
 
 			/* add workers on foremen. Also, subtract foremen from workers
 			 * connected, as they were not deployed by the pool. */
-
-			workers_needed    += count_workers_needed(foremen_list, 1);
+			workers_needed    += count_workers_needed(foremen_list, 0);
 			workers_connected += MAX(count_workers_connected(foremen_list) - list_size(foremen_list), 0);
 
 			debug(D_WQ,"%d total workers needed across %d foremen",workers_needed,list_size(foremen_list));
+		} else {
+			/* If there are no foremen, workers needed are computed directly
+			 * from the tasks running, waiting, and left from the masters'
+			 * list. */
+			workers_needed = count_workers_needed(masters_list, 0);
+			debug(D_WQ,"%d total workers needed across %d masters",
+					workers_needed,
+					masters_list ? list_size(masters_list) : 0);
 		}
 
 		debug(D_WQ,"raw workers needed: %d", workers_needed);
@@ -1003,6 +1022,7 @@ static void show_help(const char *cmd)
 	printf(" %-30s Specify the size of the debug file (must use with -o option).\n", "-O,--debug-file-size=<mb>");
 	printf(" %-30s Specify the binary to use for the worker (relative or hard path). It should accept the same arguments as the default work_queue_worker.\n", "--worker-binary=<file>");
 	printf(" %-30s Will make a best attempt to ensure the worker will execute in the specified OS environment, regardless of the underlying OS.\n","--runos=<img>");
+	printf(" %-30s Force factory to run itself as a work queue master.\n","--run-factory-as-master");
 	printf(" %-30s Show the version string.\n", "-v,--version");
 	printf(" %-30s Show this screen.\n", "-h,--help");
 }
@@ -1028,6 +1048,7 @@ enum{   LONG_OPT_CORES = 255,
 		LONG_OPT_K8S_WORKER_IMAGE,
 		LONG_OPT_CATALOG,
 		LONG_OPT_ENVIRONMENT_VARIABLE,
+		LONG_OPT_RUN_AS_MASTER,
 		LONG_OPT_RUN_OS,
 	};
 
@@ -1061,6 +1082,7 @@ static const struct option long_options[] = {
 	{"mesos-preload", required_argument, 0, LONG_OPT_MESOS_PRELOAD},
 	{"min-workers", required_argument, 0, 'w'},
 	{"password", required_argument, 0, 'P'},
+	{"run-factory-as-master", no_argument, 0, LONG_OPT_RUN_AS_MASTER},
 	{"runos", required_argument, 0, LONG_OPT_RUN_OS},
 	{"scratch-dir", required_argument, 0, 'S' },
 	{"tasks-per-worker", required_argument, 0, LONG_OPT_TASKS_PER_WORKER},
@@ -1080,6 +1102,10 @@ int main(int argc, char *argv[])
 	char *mesos_path = NULL;
 	char *mesos_preload = NULL;
 	char *k8s_image = NULL;
+	struct jx *wrapper_inputs = jx_array(NULL);
+
+	// --run-factory-as-master and -Twq should be used together.
+	int run_as_master = 0;
 
 	//Environment variable handling
 	char *ev = NULL;
@@ -1195,9 +1221,11 @@ int main(int argc, char *argv[])
 				} else {
 					wrapper_input = string_format("%s,%s",wrapper_input,optarg);
 				}
+				struct jx *file = jx_string(optarg);
+				jx_array_append(wrapper_inputs, file);
 				break;
 			case LONG_OPT_WORKER_BINARY:
-				worker_command = strdup(optarg);
+				worker_command = xxstrdup(optarg);
 				break;
 			case 'P':
 				password_file = optarg;
@@ -1242,6 +1270,9 @@ int main(int argc, char *argv[])
 			case LONG_OPT_CATALOG:
 				catalog_host = xxstrdup(optarg);
 				break;
+			case LONG_OPT_RUN_AS_MASTER:
+				run_as_master = 1;
+				break;
 			case LONG_OPT_RUN_OS:
 				runos_os = xxstrdup(optarg);
 				break;
@@ -1251,6 +1282,16 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if(batch_queue_type == BATCH_QUEUE_TYPE_WORK_QUEUE && !run_as_master) {
+		fprintf(stderr, "work_queue_factory: batch system 'wq' specified, but you most likely want 'local'.\n");
+		fprintf(stderr, "work_queue_factory: use --run-factory-as-master if you know what you are doing.\n");
+		exit(EXIT_FAILURE);
+	} else if(batch_queue_type != BATCH_QUEUE_TYPE_WORK_QUEUE && run_as_master) {
+		fprintf(stderr, "work_queue_factory: --run-factory-as-master can only be used with -Twq.\n");
+		fprintf(stderr, "work_queue_factory: you most likely want -Tlocal instead.\n");
+		exit(EXIT_FAILURE);
+	}
+
 	if(config_file) {
 		char abs_path_name[PATH_MAX];
 
@@ -1284,7 +1325,7 @@ int main(int argc, char *argv[])
 		show_help(argv[0]);
 		exit(1);
 	}
-	
+
 	cctools_version_debug(D_DEBUG, argv[0]);
 
 	if(batch_queue_type == BATCH_QUEUE_TYPE_UNKNOWN) {
@@ -1331,24 +1372,39 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	if(wrapper_input) {
+		struct jx *item;
+		for(void *i = NULL; (item = jx_iterate_array(wrapper_inputs, &i));) {
+			const char *value = item->u.string_value;
+			const char *file_at_scratch_dir = string_format("%s/%s", scratch_dir, path_basename(value));
+			int result = copy_direntry(value, file_at_scratch_dir); 
+			if(result < 0) {
+				debug(D_NOTICE, "Cannot copy wrapper input file %s to factory scratch directory", value);
+			}
+		}
+	}
+
 	char* cmd;
 	if(worker_command != NULL){
 		cmd = string_format("cp '%s' '%s'",worker_command,scratch_dir);
 		if(system(cmd)){
-			fprintf(stderr, "work_queue_factory: Could not Access specified worker_queue_worker binary.\n");
+			fprintf(stderr, "work_queue_factory: Could not Access specified worker binary.\n");
 			exit(EXIT_FAILURE);
 		}
 		free(cmd);
+		char *tmp = xxstrdup(path_basename(worker_command));
+		free(worker_command);
+		worker_command = tmp;
 	}else{
-		cmd = string_format("cp \"$(which work_queue_worker)\" '%s'",scratch_dir);
+		worker_command = xxstrdup("work_queue_worker");
+		cmd = string_format("cp \"$(which %s)\" '%s'",worker_command,scratch_dir);
 		if (system(cmd)) {
 			fprintf(stderr, "work_queue_factory: please add work_queue_worker to your PATH.\n");
 			exit(EXIT_FAILURE);
 		}
 		free(cmd);
 	}
-	
-	
+
 	if(runos_os) {
 		cmd = string_format("cp '%s' '%s'",  CCTOOLS_VC3_BUILDER_PATH, scratch_dir);
 		int k = system(cmd);
@@ -1414,6 +1470,7 @@ int main(int argc, char *argv[])
 
 	}
 
+	jx_delete(wrapper_inputs);
 	batch_queue_delete(queue);
 
 	return 0;
