@@ -40,13 +40,10 @@ See the file COPYING for details.
 #ifdef HAS_SYS_STATVFS_H
 #	include <sys/statvfs.h>
 #endif
-#if CCTOOLS_OPSYS_CYGWIN || CCTOOLS_OPSYS_DARWIN || CCTOOLS_OPSYS_FREEBSD
+#ifdef CCTOOLS_OPSYS_DARWIN
 #	include <sys/mount.h>
 #	include <sys/param.h>
 #	define statfs64 statfs
-#elif CCTOOLS_OPSYS_SUNOS
-#	define statfs statvfs
-#	define statfs64 statvfs64
 #endif
 
 #include <assert.h>
@@ -55,7 +52,7 @@ See the file COPYING for details.
 #include <string.h>
 #include <time.h>
 
-#if CCTOOLS_OPSYS_CYGWIN || CCTOOLS_OPSYS_DARWIN || CCTOOLS_OPSYS_FREEBSD || CCTOOLS_OPSYS_DRAGONFLY
+#ifdef CCTOOLS_OPSYS_DARWIN
 	/* Cygwin does not have 64-bit I/O, while FreeBSD/Darwin has it by default. */
 #	define stat64 stat
 #	define fstat64 fstat
@@ -63,12 +60,6 @@ See the file COPYING for details.
 #	define statfs64 statfs
 #	define fstatfs64 fstatfs
 #	define fstatat64 fstatat
-#elif defined(CCTOOLS_OPSYS_SUNOS)
-	/* Solaris has statfs, but it doesn't work! Use statvfs instead. */
-#	define statfs statvfs
-#	define fstatfs fstatvfs
-#	define statfs64 statvfs64
-#	define fstatfs64 fstatvfs64
 #endif
 
 #define TICKET_REFRESH (6*60*60)
