@@ -8,13 +8,12 @@ set -e
 c="./hostport.$PPID"
 cr="./root.$PPID"
 
+python=${CCTOOLS_PYTHON_TEST_EXEC}
+python_dir=${CCTOOLS_PYTHON_TEST_DIR}
 
 check_needed()
 {
-	[ -f ../src/bindings/python/_CChirp.so ] || return 1
-	python=$(cctools_python -n 3 2.7 2.6)
-	[ -n "$python" ] || return 1
-	${python} -c "import json; import Chirp"
+	[ -n "${python}" ] || return 1
 }
 
 prepare()
@@ -32,7 +31,9 @@ run()
 	fi
 	hostport=$(cat "$c")
 
-	../src/bindings/python/chirp_jobs_python_example.py $hostport ../src/bindings/python/my_script.sh
+	base=$(pwd)/../src/bindings/${python_dir}
+
+	PYTHONPATH=${base} ${python} ${base}/chirp_jobs_python_example.py $hostport ${base}/my_script.sh
 
 	return 0
 }
