@@ -9,9 +9,11 @@ c="./hostport.$PPID"
 
 ticket=my.ticket
 
+perl=${CCTOOLS_PERL}
+
 check_needed()
 {
-	test -f ../src/bindings/perl/CChirp.so
+	[ -n "${perl}" ] || return 1
 	command -v openssl >/dev/null 2>&1
 }
 
@@ -31,7 +33,7 @@ run()
 
 	chirp -d all -a unix "$hostport" ticket_create -output "$ticket" -bits 1024 -duration 86400 -subject unix:`whoami` / write
 
-	PERL5LIB=$(pwd)/../src/bindings/perl ../src/bindings/perl/chirp_perl_example.pl $hostport $ticket
+	PERL5LIB="$(pwd)/../src/bindings/perl" ${perl} ../src/bindings/perl/chirp_perl_example.pl $hostport $ticket
 
 	return 0
 }
