@@ -9,16 +9,18 @@ c="./hostport.$PPID"
 
 ticket=my.ticket
 
+python=${CCTOOLS_PYTHON_TEST_EXEC}
+python_dir=${CCTOOLS_PYTHON_TEST_DIR}
+
 check_needed()
 {
-	[ -f ../src/bindings/python/_CChirp.so ] || return 1
-	python=$(cctools_python -n 3 2.7 2.6)
-	[ -n "$python" ] || return 1
-	${python} -c "import json; import Chirp"
+	[ -n "${python}" ] || return 1
 }
 
 prepare()
 {
+	[ -n "$python" ] || return 1
+
 	chirp_start local --auth=ticket
 	echo "$hostport" > "$c"
 	return 0
@@ -33,7 +35,9 @@ run()
 
 	chirp -d all -a unix "$hostport" ticket_create -output "$ticket" -bits 1024 -duration 86400 -subject unix:`whoami` / write
 
-	../src/bindings/python/chirp_python_example.py $hostport $ticket
+	base=$(pwd)/../src/bindings/${python_dir}/
+	PYTHONPATH=${base} ${python} ${base}/chirp_python_example.py $hostport $ticket
+
 
 	return 0
 }
