@@ -61,15 +61,17 @@ check_file ()
 
 check_path ()
 {
-	echon "checking for $1..."
+	echon "checking for executable $1..."
 
 	local check_path_var=$1
 	if [ "${check_path_var}" != "${check_path_var#/}" ]
 	then
-		if check_file $check_path_var
+		if [ -f "${check_path_var}" -a -x "${check_path_var}" ]
 		then
+			echo "yes"
 			return 0
 		else
+			echo "no"
 			return 1
 		fi
 	fi
@@ -77,18 +79,34 @@ check_path ()
 	IFS=":"
 	for dir in $PATH
 	do
-		if [ -x $dir/$1 ]
+		if [ -f $dir/$1 -a -x $dir/$1 ]
 		then
-			echo "$dir/$1"
+			echo $dir/$1
 			IFS=" "
 			return 0
 		fi
 	done
-	echo "not found"
+	echo "no"
 	IFS=" "
 	return 1
 }
 
+search_file_executable() {
+	executable_search_result=""
+	for candidate in "$@"
+	do
+		if check_path ${candidate}
+		then
+			executable_search_result=$(which ${candidate})
+			return 0
+		fi
+	done
+
+	return 1
+}
+
+
+
 check_library_static() {
 	local libdir
 	local library
@@ -128,7 +146,12 @@ check_function()
 	local header
 
 	name="$1"
-	header="$2"
+	shift
+
+	header="$1"
+	shift
+
+	compiler_options="$@"
 
 	echon "checking for $name in $header..."
 
@@ -140,7 +163,7 @@ int main(int argc, char **argv) {
 	return 0;
 }
 EOF
-	if ${CC:-gcc} .configure.tmp.c -c -o .configure.tmp.o > .configure.tmp.out 2>&1; then
+	if ${CC:-gcc} .configure.tmp.c -c -o .configure.tmp.o ${compiler_options} > .configure.tmp.out 2>&1; then
 		echo yes
 		rm -f .configure.tmp.c .configure.tmp.out
 		return 0
@@ -178,9 +201,19 @@ require_file ()
 	fi
 }
 
-require_path ()
+optional_path ()
 {
 	if check_path $1
+	then
+		return 0
+	else
+		return 1
+	fi
+}
+
+require_path ()
+{
+	if optional_path $1
 	then
 		return 0
 	else
@@ -672,4 +705,27 @@ config_X_path()
 	fi
 }
 
+report_detection()
+{
+	package=$1
+	available=$2
+	required=$3
+	path_tried=$4
+
+	if [ "${available}" = yes ]
+	then
+		echo "compiling with ${package} support"
+	elif [ "${required}" = no ]
+	then
+		echo "${package} support explicitely disabled"
+	elif [ "${required}" = yes ]
+	then
+		echo "*** Sorry, I couldn't find ${package} in ${path_tried}"
+		echo "*** Check --with-${package}-path and try again"
+		exit 1
+	else
+		echo "skipping ${package} support as it could not be automatically detected"
+	fi
+}
+
 # vim: set noexpandtab tabstop=4:
