@@ -65,8 +65,13 @@ struct jx * jx_symbol( const char *symbol_name )
 struct jx * jx_string( const char *string_value )
 {
 	assert(string_value);
+	return jx_string_nocopy(strdup(string_value));
+}
+
+struct jx * jx_string_nocopy( char *string_value )
+{
 	struct jx *j = jx_create(JX_STRING);
-	j->u.string_value = strdup(string_value);
+	j->u.string_value = string_value;
 	return j;
 }
 
@@ -143,17 +148,6 @@ struct jx * jx_error( struct jx *err )
 	return j;
 }
 
-struct jx *jx_function(const char *name, jx_builtin_t op,
-	struct jx_item *params, struct jx *body) {
-	assert(name);
-	struct jx *j = jx_create(JX_FUNCTION);
-	j->u.func.name = strdup(name);
-	j->u.func.params = params;
-	j->u.func.body = body;
-	j->u.func.builtin = op;
-	return j;
-}
-
 struct jx * jx_arrayv( struct jx *value, ... )
 {
 	va_list args;
@@ -420,11 +414,6 @@ void jx_delete( struct jx *j )
 			jx_delete(j->u.oper.left);
 			jx_delete(j->u.oper.right);
 			break;
-		case JX_FUNCTION:
-			free(j->u.func.name);
-			jx_item_delete(j->u.func.params);
-			jx_delete(j->u.func.body);
-			break;
 		case JX_ERROR:
 			jx_delete(j->u.err);
 			break;
@@ -507,11 +496,6 @@ int jx_equals( struct jx *j, struct jx *k )
 			return j->u.oper.type == k->u.oper.type
 				&& jx_equals(j->u.oper.left,k->u.oper.right)
 				&& jx_equals(j->u.oper.right,j->u.oper.right);
-		case JX_FUNCTION:
-			return !strcmp(j->u.func.name, k->u.func.name)
-				&& jx_item_equals(
-					   j->u.func.params, k->u.func.params)
-				&& jx_equals(j->u.func.body, k->u.func.body);
 		case JX_ERROR:
 			return jx_equals(j->u.err, k->u.err);
 	}
@@ -556,7 +540,7 @@ struct jx_item * jx_item_copy( struct jx_item *i )
 struct jx  *jx_copy( struct jx *j )
 {
 	if(!j) return 0;
-	struct jx *c;
+	struct jx *c = 0;
 
 	switch(j->type) {
 		case JX_NULL:
@@ -586,17 +570,15 @@ struct jx  *jx_copy( struct jx *j )
 		case JX_OPERATOR:
 			c = jx_operator(j->u.oper.type, jx_copy(j->u.oper.left), jx_copy(j->u.oper.right));
 			break;
-		case JX_FUNCTION:
-			c = jx_function(j->u.func.name, j->u.func.builtin,
-				jx_item_copy(j->u.func.params),
-				jx_copy(j->u.func.body));
-			break;
 		case JX_ERROR:
 			c = jx_error(jx_copy(j->u.err));
 			break;
 	}
 
-	c->line = j->line;
+    if(c) {
+        c->line = j->line;
+    }
+
 	return c;
 }
 
@@ -644,7 +626,6 @@ int jx_is_constant( struct jx *j )
 		case JX_OBJECT:
 			return jx_pair_is_constant(j->u.pairs);
 		case JX_ERROR:
-		case JX_FUNCTION:
 		case JX_OPERATOR:
 			return 0;
 	}
