@@ -49,7 +49,6 @@ typedef enum {
 	JX_ARRAY,    /**< array containing values */
 	JX_OBJECT,   /**< object containing key-value pairs */
 	JX_OPERATOR, /**< operator on multiple values. */
-	JX_FUNCTION, /**< function definition */
 	JX_ERROR,    /**< indicates failed evaluation */
 } jx_type_t;
 
@@ -109,27 +108,6 @@ struct jx_operator {
 	struct jx *right;
 };
 
-typedef enum {
-	JX_BUILTIN_LAMBDA,
-	JX_BUILTIN_RANGE,
-	JX_BUILTIN_FORMAT,
-	JX_BUILTIN_JOIN,
-	JX_BUILTIN_CEIL,
-	JX_BUILTIN_FLOOR,
-	JX_BUILTIN_BASENAME,
-	JX_BUILTIN_DIRNAME,
-	JX_BUILTIN_LISTDIR,
-	JX_BUILTIN_ESCAPE,
-} jx_builtin_t;
-
-struct jx_function {
-	char *name;
-	unsigned line;
-	struct jx_item *params;
-	struct jx *body;
-	jx_builtin_t builtin;
-};
-
 /** JX value representing any expression type. */
 
 struct jx {
@@ -144,7 +122,6 @@ struct jx {
 		struct jx_item *items;  /**< value of @ref JX_ARRAY */
 		struct jx_pair *pairs;  /**< value of @ref JX_OBJECT */
 		struct jx_operator oper; /**< value of @ref JX_OPERATOR */
-		struct jx_function func; /**< value of @ref JX_FUNCTION */
 		struct jx *err;  /**< error value of @ref JX_ERROR */
 	} u;
 };
@@ -161,9 +138,12 @@ struct jx * jx_integer( jx_int_t integer_value );
 /** Create a JX floating point value. @param double_value A C double precision floating point.  @return a JX double value. */
 struct jx * jx_double( double double_value );
 
-/** Create a JX string value. @param string_value A C string, which will be duplciated via strdup(). @return A JX string value. */
+/** Create a JX string value. @param string_value A C string, which will be duplicated via strdup(). @return A JX string value. */
 struct jx * jx_string( const char *string_value );
 
+/** Create a JX string value without copying (uncommon). @param string_value A C string, which will be *not* be duplicated, but will be freed at object deletion.  @return A JX string value. */
+struct jx * jx_string_nocopy( char *string_value );
+
 /** Create a JX string value using prinf style formatting.  @param fmt A printf-style format string, followed by matching arguments.  @return A JX string value. */
 struct jx * jx_format( const char *fmt, ... );
 
@@ -174,12 +154,6 @@ struct jx * jx_symbol( const char *symbol_name );
 /** Create a JX_ERROR. @param err The associated data for the error. This should be a string description of the error. @return A JX error value. */
 struct jx * jx_error( struct jx *err );
 
-/** Create a JX_FUNCTION. @param params The list of JX_STRING parameter names.
- * @param body The function body to evaluate. @returns A JX function definition.
- */
-struct jx *jx_function(const char *name, jx_builtin_t op,
-	struct jx_item *params, struct jx *body);
-
 /** Create a JX array.  @param items A linked list of @ref jx_item values.  @return A JX array. */
 struct jx * jx_array( struct jx_item *items );
 
@@ -386,3 +360,5 @@ struct jx *jx_get_value(void **i);
 struct jx *jx_merge(struct jx *j, ...);
 
 #endif
+
+/*vim: set noexpandtab tabstop=4: */
