@@ -18,5 +18,7 @@ struct jx *jx_function_basename(struct jx *args);
 struct jx *jx_function_dirname(struct jx *args);
 struct jx *jx_function_listdir(struct jx *args);
 struct jx *jx_function_escape(struct jx *args);
+struct jx *jx_function_template(struct jx *args, struct jx *ctx);
+struct jx *jx_function_len(struct jx *args);
 
 #endif
