@@ -153,7 +153,7 @@ void jx_print_buffer( struct jx *j, buffer_t *b )
 			buffer_putstring(b,"null");
 			break;
 		case JX_DOUBLE:
-			buffer_printf(b,"%g",j->u.double_value);
+			buffer_printf(b,"%.16g",j->u.double_value);
 			break;
 		case JX_BOOLEAN:
 			buffer_printf(b,"%s",j->u.boolean_value ? "true" : "false");
@@ -189,7 +189,6 @@ void jx_print_buffer( struct jx *j, buffer_t *b )
 			}
 			if(j->u.oper.type==JX_OP_LOOKUP) buffer_putstring(b,"]");
 			break;
-		case JX_FUNCTION: buffer_putstring(b, j->u.func.name); break;
 		case JX_ERROR:
 			buffer_putstring(b, "error(");
 			jx_print_buffer(j->u.err, b);
