@@ -55,7 +55,6 @@ See the file COPYING for details.
 #include "parser_jx.h"
 #include "parser.h"
 
-
 /* Returns a pointer to a new struct dag described by filename. Return NULL on
  * failure. */
 struct dag *dag_from_file(const char *filename, dag_syntax_type format, struct jx *args)
@@ -185,6 +184,11 @@ void rmsummary_set_resources_from_env(struct rmsummary *rs, struct dag_variable_
 	if(val) {
 		rs->gpus = atoll(val->value);
 	}
+
+	val = dag_variable_lookup(RESOURCES_WALL_TIME, &s);
+	if(val) {
+		rs->wall_time = atoll(val->value);
+	}
 }
 
 void dag_close_over_nodes(struct dag *d)
