@@ -40,6 +40,7 @@ extern "C" {
 #include "getopt.h"
 #include "int_sizes.h"
 #include "itable.h"
+#include "macros.h"
 #include "md5.h"
 #include "password_cache.h"
 #include "random.h"
@@ -72,6 +73,8 @@ extern "C" {
 #include <string.h>
 #include <time.h>
 
+#include <vector>
+
 #define SIG_ISSTOP(s) (s == SIGTTIN || s == SIGTTOU || s == SIGSTOP || s == SIGTSTP)
 
 FILE *namelist_file;
@@ -126,6 +129,7 @@ INT64_T pfs_write_count = 0;
 
 const char * pfs_cvmfs_repo_arg = 0;
 const char * pfs_cvmfs_config_arg = 0;
+const char * pfs_cvmfs_http_proxy = 0;
 bool pfs_cvmfs_repo_switching = false;
 char pfs_cvmfs_alien_cache_dir[PATH_MAX];
 char pfs_cvmfs_locks_dir[PATH_MAX];
@@ -133,12 +137,15 @@ bool pfs_cvmfs_enable_alien  = true;
 char pfs_cvmfs_option_file[PATH_MAX];
 struct jx *pfs_cvmfs_options = NULL;
 
+
 int pfs_irods_debug_level = 0;
 char *stats_file = NULL;
 
 int parrot_fd_max = -1;
 int parrot_fd_start = -1;
 
+pfs_service *pfs_service_ext_init(const char *image, const char *mountpoint);
+
 /*
 This process at the very top of the traced tree
 and its final exit status, which we use to determine
@@ -172,6 +179,7 @@ enum {
 	LONG_OPT_STATS_FILE,
 	LONG_OPT_DISABLE_SERVICE,
 	LONG_OPT_NO_FLOCK,
+	LONG_OPT_EXT_IMAGE,
 };
 
 static void get_linux_version(const char *cmd)
@@ -282,6 +290,8 @@ static void show_help( const char *cmd )
 	printf( " %-30s Disable the given service.\n", "--disable-service");
 	printf( " %-30s Make flock a no-op.\n", "--no-flock");
 	printf("\n");
+	printf("Filesystem Options:\n");
+	printf( " %-30s Mount a read-only ext[234] disk image.\n", "--ext <image>=<mountpoint>");
 	printf("FTP / GridFTP options:\n");
 	printf( " %-30s Enable data channel authentication in GridFTP.\n", "-C,--channel-auth");
 	printf("\n");
@@ -588,13 +598,13 @@ int main( int argc, char *argv[] )
 	int c;
 	int chose_auth = 0;
 	char *tickets = NULL;
-	char *http_proxy = NULL;
 	pid_t pid;
 	struct pfs_process *p;
 	char envlist[PATH_MAX] = "";
 	int valgrind = 0;
 	int envdebug = 0;
 	int envauth = 0;
+	std::vector<pfs_service *> service_instances;
 
 	random_init();
 	pfs_resolve_init();
@@ -784,7 +794,7 @@ int main( int argc, char *argv[] )
 		snprintf(pfs_temp_dir, sizeof(pfs_temp_dir), "%s", s);
 	} else {
 		assert(sys_temp_dir[0]);
-		snprintf(pfs_temp_dir, sizeof(pfs_temp_dir), "%s/parrot.%d", sys_temp_dir, getuid());
+		string_nformat(pfs_temp_dir, sizeof(pfs_temp_dir), "%s/parrot.%d", sys_temp_dir, getuid());
 	}
 
 	s = getenv("PARROT_CVMFS_ALIEN_CACHE");
@@ -792,12 +802,12 @@ int main( int argc, char *argv[] )
 		snprintf(pfs_cvmfs_alien_cache_dir, sizeof(pfs_cvmfs_alien_cache_dir), "%s", s);
 	} else {
 		assert(pfs_temp_dir[0]);
-		snprintf(pfs_cvmfs_alien_cache_dir, sizeof(pfs_cvmfs_alien_cache_dir), "%s/cvmfs", pfs_temp_dir);
+		string_nformat(pfs_cvmfs_alien_cache_dir, sizeof(pfs_cvmfs_alien_cache_dir), "%s/cvmfs", pfs_temp_dir);
 	}
 
 	s = getenv("PARROT_CVMFS_OPTION_FILE");
 	if (s) {
-		snprintf(pfs_cvmfs_option_file, sizeof(pfs_cvmfs_option_file), "%s", s);
+		string_nformat(pfs_cvmfs_option_file, sizeof(pfs_cvmfs_option_file), "%s", s);
 	} else {
 		memset(pfs_cvmfs_option_file, 0, sizeof(pfs_cvmfs_option_file));
 	}
@@ -819,8 +829,10 @@ int main( int argc, char *argv[] )
 		{"debug-file", required_argument, 0, 'o'},
 		{"debug-level-irods", required_argument, 0, 'I'},
 		{"debug-rotate-max", required_argument, 0, 'O'},
+		{"disable-service", required_argument, 0, LONG_OPT_DISABLE_SERVICE},
 		{"dynamic-mounts", no_argument, 0, LONG_OPT_DYNAMIC_MOUNTS },
 		{"env-list", required_argument, 0, 'e'},
+		{"ext-image", required_argument, 0, LONG_OPT_EXT_IMAGE},
 		{"fake-setuid", no_argument, 0, LONG_OPT_FAKE_SETUID},
 		{"gid", required_argument, 0, 'G'},
 		{"help", no_argument, 0, 'h'},
@@ -838,6 +850,8 @@ int main( int argc, char *argv[] )
 		{"no-set-foreground", no_argument, 0, LONG_OPT_NO_SET_FOREGROUND},
 		{"paranoid", no_argument, 0, 'P'},
 		{"parrot-path", required_argument, 0, LONG_OPT_PARROT_PATH},
+		{"pid-fixed", no_argument, 0, LONG_OPT_PID_FIXED},
+		{"pid-warp", no_argument, 0, LONG_OPT_PID_WARP},
 		{"proxy", required_argument, 0, 'p'},
 		{"root-checksum", required_argument, 0, 'R'},
 		{"session-caching", no_argument, 0, 'S'},
@@ -850,6 +864,8 @@ int main( int argc, char *argv[] )
 		{"tab-file", required_argument, 0, 'm'},
 		{"tempdir", required_argument, 0, 't'},
 		{"tickets", required_argument, 0, 'i'},
+		{"time-stop", no_argument, 0, LONG_OPT_TIME_STOP},
+		{"time-warp", no_argument, 0, LONG_OPT_TIME_WARP},
 		{"timeout", required_argument, 0, 'T'},
 		{"uid", required_argument, 0, 'U'},
 		{"username", required_argument, 0, 'u'},
@@ -859,11 +875,6 @@ int main( int argc, char *argv[] )
 		{"with-checksums", no_argument, 0, 'K'},
 		{"with-snapshots", no_argument, 0, 'F'},
 		{"work-dir", required_argument, 0, 'w'},
-		{"time-stop", no_argument, 0, LONG_OPT_TIME_STOP},
-		{"time-warp", no_argument, 0, LONG_OPT_TIME_WARP},
-		{"pid-fixed", no_argument, 0, LONG_OPT_PID_FIXED},
-		{"pid-warp", no_argument, 0, LONG_OPT_PID_WARP},
-		{"disable-service", required_argument, 0, LONG_OPT_DISABLE_SERVICE},
 		{0,0,0,0}
 	};
 
@@ -973,7 +984,7 @@ int main( int argc, char *argv[] )
 			debug_config_file_size(string_metric_parse(optarg));
 			break;
 		case 'p':
-			http_proxy = xxstrdup(optarg);
+			pfs_cvmfs_http_proxy = xxstrdup(optarg);
 			break;
 		case 'P':
 			pfs_paranoid_mode = 1;
@@ -1074,6 +1085,7 @@ int main( int argc, char *argv[] )
 		case LONG_OPT_FAKE_SETUID:
 			pfs_fake_setuid = 1;
 			pfs_fake_setgid = 1;
+			break;
 		case LONG_OPT_DYNAMIC_MOUNTS:
 			pfs_allow_dynamic_mounts = 1;
 			break;
@@ -1118,6 +1130,27 @@ int main( int argc, char *argv[] )
 		case LONG_OPT_NO_FLOCK:
 			pfs_no_flock = 1;
 			break;
+		case LONG_OPT_EXT_IMAGE: {
+			char service[128];
+			char image[PATH_MAX] = {0};
+			char mountpoint[PATH_MAX] = {0};
+			static unsigned ext_no = 0;
+
+			char *split = strchr(optarg, '=');
+			if (!split) fatal("--ext must be specified as IMAGE=MOUNTPOINT");
+			strncpy(image, optarg, MIN((size_t) (split - optarg), sizeof(image) - 1));
+			strncpy(mountpoint, split + 1, sizeof(mountpoint) - 1);
+			if (mountpoint[0] != '/') fatal("mountpoint for ext image %s must be an absolute path", image);
+			struct pfs_service *s = pfs_service_ext_init(image, mountpoint);
+			if (!s) fatal("failed to load ext image %s", image);
+
+			service_instances.push_back(s);
+			snprintf(service, sizeof(service), "/ext_%u", ext_no++);
+			hash_table_insert(available_services, &service[1], s);
+			pfs_resolve_add_entry(mountpoint, service, R_OK|W_OK|X_OK);
+
+			break;
+		}
 		default:
 			show_help(argv[0]);
 			break;
@@ -1126,7 +1159,7 @@ int main( int argc, char *argv[] )
 
 	if(optind>=argc) show_help(argv[0]);
 
-	FILE *stats_out;
+	FILE *stats_out = NULL;
 	if (stats_file) {
 		stats_enable();
 		stats_out = fopen(stats_file, "w");
@@ -1184,15 +1217,16 @@ int main( int argc, char *argv[] )
 		fclose(fp);
 	}
 
-	if (http_proxy)
-		setenv("PARROT_HTTP_PROXY", http_proxy, 1);
-
-	http_proxy = (char *)realloc(http_proxy, 0);
+	// if -p not given, check if HTTP_PROXY is set.
+	if(!pfs_cvmfs_http_proxy && getenv("HTTP_PROXY")) {
+		pfs_cvmfs_http_proxy = xxstrdup(getenv("HTTP_PROXY"));
+	}
 
 	if (!create_dir(pfs_temp_dir, S_IRWXU))
 		fatal("could not create directory '%s': %s", pfs_temp_dir, strerror(errno));
 
-	snprintf(pfs_temp_per_instance_dir, sizeof(pfs_temp_per_instance_dir), "%s/parrot-instance.XXXXXX", pfs_temp_dir);
+	string_nformat(pfs_temp_per_instance_dir, sizeof(pfs_temp_per_instance_dir), "%s/parrot-instance.XXXXXX", pfs_temp_dir);
+
 	if (mkdtemp(pfs_temp_per_instance_dir) == NULL)
 		fatal("could not create directory '%s': %s", pfs_temp_per_instance_dir, strerror(errno));
 
@@ -1200,7 +1234,8 @@ int main( int argc, char *argv[] )
 	if(!pfs_file_cache) fatal("couldn't setup cache in %s: %s\n",pfs_temp_dir,strerror(errno));
 	file_cache_cleanup(pfs_file_cache);
 
-	snprintf(pfs_cvmfs_locks_dir, sizeof(pfs_cvmfs_locks_dir), "%s/cvmfs_locks_XXXXXX", pfs_temp_per_instance_dir);
+	string_nformat(pfs_cvmfs_locks_dir, sizeof(pfs_cvmfs_locks_dir), "%s/cvmfs_locks_XXXXXX", pfs_temp_per_instance_dir);
+
 	if(mkdtemp(pfs_cvmfs_locks_dir) == NULL)
 		fatal("could not create a cvmfs locks temporary directory: %s", strerror(errno));
 
@@ -1219,8 +1254,10 @@ int main( int argc, char *argv[] )
 
 	{
 		int fd;
+
 		char buf[PATH_MAX];
-		snprintf(buf, sizeof(buf), "%s/parrot-fd.XXXXXX", pfs_temp_per_instance_dir);
+		string_nformat(buf, sizeof(buf), "%s/parrot-fd.XXXXXX", pfs_temp_per_instance_dir);
+
 		if (mkdtemp(buf) == NULL)
 			fatal("could not create parrot-fd temporary directory: %s", strerror(errno));
 		fd = open(buf, O_RDONLY|O_DIRECTORY);
@@ -1384,6 +1421,10 @@ int main( int argc, char *argv[] )
 		}
 	}
 
+	for (std::vector<pfs_service *>::iterator it = service_instances.begin(); it != service_instances.end(); ++it) {
+		delete *it;
+	}
+
 	if(pfs_syscall_totals32) {
 		printf("\nParrot System Call Summary:\n");
 		printf("%" PRId64 " syscalls\n",pfs_syscall_count);
