@@ -45,6 +45,7 @@ extern "C" {
 #include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/time.h>
+#include <sys/sysmacros.h>
 
 #ifndef major
 /* glibc 2.26 drops sys/sysmacros.h from sys/types.h, thus we add it here */
@@ -526,6 +527,11 @@ int pfs_table::resolve_name(int is_special_syscall, const char *cname, struct pf
 			strcpy(pname->hostport,"localhost");
 			strcpy(pname->rest,pname->path);
 			pname->is_local = 1;
+		} else if (!strncmp(pname->service_name, "ext_", 4)) {
+			strcpy(pname->rest, tmp);
+			strcpy(pname->host, "ext");
+			strcpy(pname->hostport, "ext");
+			pname->port = 0;
 		} else {
 			if(!strcmp(pname->service_name,"multi")) {// if we're dealing with a multivolume, split off at the @
 				path_split_multi(tmp,pname->host,pname->rest);
@@ -551,7 +557,9 @@ int pfs_table::resolve_name(int is_special_syscall, const char *cname, struct pf
 				} else {
 					pname->port = pname->service->get_default_port();
 				}
-				sprintf(pname->hostport,"%s:%d",pname->host,pname->port);
+				char *hostport=string_format("%s:%d", pname->host, pname->port);
+				strncpy(pname->hostport, hostport, sizeof(pname->hostport));
+				free(hostport);
 			}
 
 			if(!strcmp(pname->service_name,"multi")) {
