@@ -3,10 +3,6 @@
 . ../../dttools/test/test_runner_common.sh
 . ./parrot-test.sh
 
-export PARROT_ALLOW_SWITCHING_CVMFS_REPOSITORIES="yes"
-export HTTP_PROXY=http://cache01.hep.wisc.edu:3128
-export PARROT_CVMFS_REPO='*.cern.ch:pubkey=<BUILTIN-cern.ch.pub>,url=http://cvmfs-stratum-one.cern.ch/cvmfs/*.cern.ch'
-
 tmp_dir_master=${PWD}/parrot_temp_dir
 tmp_dir_hitcher=${PWD}/parrot_temp_dir_hitcher
 
