@@ -127,6 +127,7 @@ See the file COPYING for details.
 #include <sys/types.h>
 
 #include "buffer.h"
+#include "catalog_query.h"
 #include "cctools.h"
 #include "copy_stream.h"
 #include "create_dir.h"
@@ -145,6 +146,8 @@ See the file COPYING for details.
 #include "xxmalloc.h"
 #include "elfheader.h"
 #include "domain_name_cache.h"
+#include "uuid.h"
+#include "random.h"
 
 #include "rmonitor.h"
 #include "rmonitor_poll_internal.h"
@@ -235,6 +238,18 @@ struct list *snapshot_labels = NULL;    /* list of labels for current snapshot.
 
 struct itable *snapshot_watch_pids;     /* pid of all processes watching files for snapshots. */
 
+
+char *catalog_task_readable_name = NULL;
+char *catalog_uuid    = NULL;
+char *catalog_hosts   = NULL;
+char *catalog_project = NULL;
+char *catalog_owner   = NULL;
+
+uint64_t catalog_interval = 0;
+uint64_t catalog_last_update_time = 0;
+
+int64_t catalog_interval_default = 30;
+
 /***
  * Utility functions (open log files, proc files, measure time)
  ***/
@@ -329,7 +344,8 @@ void parse_limit_string(struct rmsummary *limits, char *str)
 	char *delim = strchr(pair, ':');
 
 	if(!delim) {
-		fatal("Missing ':' in '%s'\n", str);
+		debug(D_FATAL, "Missing ':' in '%s'\n", str);
+		exit(RM_MONITOR_ERROR);
 	}
 
 	*delim = '\0';
@@ -343,7 +359,8 @@ void parse_limit_string(struct rmsummary *limits, char *str)
 	status = string_is_float(value, &d);
 
 	if(!status) {
-		fatal("Invalid limit field '%s' or value '%s'\n", field, value);
+		debug(D_FATAL, "Invalid limit field '%s' or value '%s'\n", field, value);
+		exit(RM_MONITOR_ERROR);
 	}
 
 	if(strcmp(field, "start") == 0 || strcmp(field, "end") == 0) {
@@ -373,7 +390,8 @@ void add_verbatim_field(const char *str) {
 	char *delim = strchr(pair, ':');
 
 	if(!delim) {
-		fatal("Missing ':' in '%s'\n", str);
+		debug(D_FATAL, "Missing ':' in '%s'\n", str);
+		exit(RM_MONITOR_ERROR);
 	}
 
 	*delim = '\0';
@@ -461,6 +479,38 @@ int rmonitor_determine_exec_type(const char *executable) {
 	return 0;
 }
 
+int send_catalog_update(struct rmsummary *s, int force) {
+
+	if(!catalog_task_readable_name) {
+		return 1;
+	}
+
+	if(!force && (timestamp_get() < catalog_last_update_time + catalog_interval * USECOND)) {
+		return 1;
+	}
+
+	struct jx *j = rmsummary_to_json(s, /* all, not only resources */ 0);
+
+	jx_insert_string(j, "type",    "task");
+	jx_insert_string(j, "uuid",    catalog_uuid);
+	jx_insert_string(j, "owner",   catalog_owner);
+	jx_insert_string(j, "task",    catalog_task_readable_name);
+	jx_insert_string(j, "project", catalog_project);
+
+	char *str = jx_print_string(j);
+
+	debug(D_RMON, "Sending resources snapshot to catalog server(s) at %s ...", catalog_hosts);
+	int status = catalog_query_send_update_conditional(catalog_hosts, str);
+
+	free(str);
+	jx_delete(j);
+
+	catalog_last_update_time = timestamp_get();
+
+	return status;
+}
+
+
 
 /***
  * Reference count for filesystems and working directories auxiliary functions.
@@ -681,7 +731,7 @@ int rmonitor_handle_inotify(void)
 
 #if defined(RESOURCE_MONITOR_USE_INOTIFY)
 	struct inotify_event *evdata = NULL;
-	struct rmonitor_file_info *finfo;
+	struct rmonitor_file_info *finfo = NULL;
 	struct stat fst;
 	char *fname;
 	int nbytes, evc, i;
@@ -1598,6 +1648,8 @@ void rmonitor_final_cleanup(int signum)
 
     status = rmonitor_final_summary();
 
+	send_catalog_update(summary, 1);
+
 	fclose(log_summary);
 
     if(log_series)
@@ -1610,51 +1662,6 @@ void rmonitor_final_cleanup(int signum)
     exit(status);
 }
 
-#define over_limit_check(tr, fld)\
-	if(resources_limits->fld > -1 && (tr)->fld > 0 && resources_limits->fld - (tr)->fld < 0)\
-	{\
-		warn(D_RMON, "Limit " #fld " broken.\n");\
-		if(!(tr)->limits_exceeded) { (tr)->limits_exceeded = rmsummary_create(-1); }\
-		(tr)->limits_exceeded->fld = resources_limits->fld;\
-	}
-
-/* return 0 means above limit, 1 means limist ok */
-int rmonitor_check_limits(struct rmsummary *tr)
-{
-	tr->limits_exceeded = NULL;
-
-	/* Consider errors as resources exhausted. Used for ENOSPC, ENFILE, etc. */
-	if(tr->last_error)
-		return 0;
-
-	if(!resources_limits)
-		return 1;
-
-	over_limit_check(tr, start);
-	over_limit_check(tr, end);
-	over_limit_check(tr, cores);
-	over_limit_check(tr, wall_time);
-	over_limit_check(tr, cpu_time);
-	over_limit_check(tr, max_concurrent_processes);
-	over_limit_check(tr, total_processes);
-	over_limit_check(tr, virtual_memory);
-	over_limit_check(tr, memory);
-	over_limit_check(tr, swap_memory);
-	over_limit_check(tr, bytes_read);
-	over_limit_check(tr, bytes_written);
-	over_limit_check(tr, bytes_received);
-	over_limit_check(tr, bytes_sent);
-	over_limit_check(tr, total_files);
-	over_limit_check(tr, disk);
-
-	if(tr->limits_exceeded) {
-		return 0;
-	}
-	else {
-		return 1;
-	}
-}
-
 /***
  * Functions that communicate with the helper library,
  * (un)tracking resources as messages arrive.
@@ -1811,7 +1818,7 @@ int rmonitor_dispatch_msg(void)
 
 	summary->last_error = msg.error;
 
-	if(!rmonitor_check_limits(summary))
+	if(!rmsummary_check_limits(summary, resources_limits))
 		rmonitor_final_cleanup(SIGTERM);
 
 	// find out if messages are urgent:
@@ -2043,6 +2050,10 @@ static void show_help(const char *cmd)
     fprintf(stdout, "%-30s See --without-disk-footprint below.\n", "");
     fprintf(stdout, "%-30s Do not measure working directory footprint. Overrides --measure-dir and --follow-chdir.\n", "--without-disk-footprint");
     fprintf(stdout, "\n");
+    fprintf(stdout, "%-30s Report measurements to catalog server with \"task\"=<task-name>.\n", "--catalog-task-name=<name>");
+    fprintf(stdout, "%-30s Set project name of catalog update to <project> (default=<task-name>).\n", "--catalog-project=<project>");
+    fprintf(stdout, "%-30s Use catalog server <catalog>. (default=catalog.cse.nd.edu:9094).\n", "--catalog=<catalog>");
+    fprintf(stdout, "%-30s Send update to catalog every <interval> seconds. (default=%" PRId64 ").\n", "--catalog-interval=<interval>", catalog_interval_default);
     fprintf(stdout, "\n");
     fprintf(stdout, "%-30s Do not pretty-print summaries.\n", "--no-pprint");
     fprintf(stdout, "\n");
@@ -2079,8 +2090,9 @@ int rmonitor_resources(long int interval /*in seconds */)
 		rmonitor_poll_all_processes_once(processes, p_acc);
 		rmonitor_poll_maps_once(processes, m_acc);
 
-		if(resources_flags->disk)
+		if(resources_flags->disk) {
 			rmonitor_poll_all_wds_once(wdirs, d_acc, MAX(1, interval/(MAX(1, hash_table_size(wdirs)))));
+		}
 
 		// rmonitor_fss_once(f); disabled until statfs fs id makes sense.
 
@@ -2089,8 +2101,9 @@ int rmonitor_resources(long int interval /*in seconds */)
 		rmonitor_find_max_tree(snapshot, resources_now);
 		rmonitor_log_row(resources_now);
 
-		if(!rmonitor_check_limits(summary))
+		if(!rmsummary_check_limits(summary, resources_limits)) {
 			rmonitor_final_cleanup(SIGTERM);
+		}
 
 		release_waiting_processes();
 
@@ -2102,6 +2115,8 @@ int rmonitor_resources(long int interval /*in seconds */)
 			snapshot->start = usecs_since_epoch();
 		}
 
+		send_catalog_update(resources_now, 0);
+
 		//If no more process are alive, break out of loop.
 		if(itable_size(processes) < 1)
 			break;
@@ -2189,6 +2204,10 @@ int main(int argc, char **argv) {
 		LONG_OPT_SNAPSHOT_FILE,
 		LONG_OPT_SNAPSHOT_WATCH_CONF,
 		LONG_OPT_STOP_SHORT_RUNNING,
+		LONG_OPT_CATALOG_TASK_READABLE_NAME,
+		LONG_OPT_CATALOG_SERVER,
+		LONG_OPT_CATALOG_PROJECT,
+		LONG_OPT_CATALOG_INTERVAL,
 		LONG_OPT_PID
 	};
 
@@ -2221,6 +2240,11 @@ int main(int argc, char **argv) {
 		    {"snapshot-file",   required_argument, 0, LONG_OPT_SNAPSHOT_FILE},
 		    {"snapshot-events", required_argument, 0, LONG_OPT_SNAPSHOT_WATCH_CONF},
 
+		    {"catalog-task-name", required_argument, 0, LONG_OPT_CATALOG_TASK_READABLE_NAME},
+		    {"catalog",           required_argument, 0, LONG_OPT_CATALOG_SERVER},
+		    {"catalog-project",   required_argument, 0, LONG_OPT_CATALOG_PROJECT},
+		    {"catalog-interval",  required_argument, 0, LONG_OPT_CATALOG_INTERVAL},
+
 		    {0, 0, 0, 0}
 	    };
 
@@ -2302,7 +2326,8 @@ int main(int argc, char **argv) {
 				pprint_summaries = 0;
 				break;
 			case LONG_OPT_SNAPSHOT_FILE:
-				fatal("This option has been replaced with --snapshot-events. Please consult the manual of resource_monitor.");
+				debug(D_FATAL, "This option has been replaced with --snapshot-events. Please consult the manual of resource_monitor.");
+				exit(RM_MONITOR_ERROR);
 				break;
 			case LONG_OPT_SNAPSHOT_WATCH_CONF:
 				snapshot_watch_events_file = xxstrdup(optarg);
@@ -2318,6 +2343,21 @@ int main(int argc, char **argv) {
 					first_process_pid = (pid_t) p;
 				}
 				break;
+			case LONG_OPT_CATALOG_TASK_READABLE_NAME:
+				catalog_task_readable_name = xxstrdup(optarg);
+				break;
+			case LONG_OPT_CATALOG_SERVER:
+				catalog_hosts = xxstrdup(optarg);
+				break;
+			case LONG_OPT_CATALOG_PROJECT:
+				catalog_project = xxstrdup(optarg);
+				break;
+			case LONG_OPT_CATALOG_INTERVAL:
+				catalog_interval = atoi(optarg);
+				if(catalog_interval < 1) {
+					debug(D_FATAL, "--catalog-interval cannot be less than 1.");
+				}
+				break;
 			default:
 				show_help(argv[0]);
 				return 1;
@@ -2325,7 +2365,7 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	if( follow_chdir && hash_table_size(wdirs) > 0) {
+	if(follow_chdir && hash_table_size(wdirs) > 0) {
 		debug(D_FATAL, "Options --follow-chdir and --measure-dir as mutually exclusive.");
 		exit(RM_MONITOR_ERROR);
 	}
@@ -2342,6 +2382,44 @@ int main(int argc, char **argv) {
 		}
 	}
 
+	if(catalog_task_readable_name) {
+
+		random_init();
+		cctools_uuid_t *uuid = malloc(sizeof(*uuid));
+		cctools_uuid_create(uuid);
+		catalog_uuid = xxstrdup(uuid->str);
+		free(uuid);
+
+		char *tmp = getenv("USER");
+		if(tmp) {
+			catalog_owner = xxstrdup(tmp);
+		} else {
+			catalog_owner = "unknown";
+		}
+
+		if(!catalog_hosts) {
+			catalog_hosts = xxstrdup(CATALOG_HOST);
+		}
+
+		if(!catalog_project) {
+			catalog_project = xxstrdup(catalog_task_readable_name);
+		}
+
+		if(catalog_interval < 1) {
+			catalog_interval = catalog_interval_default;
+		}
+
+		if(catalog_interval < interval) {
+			warn(D_RMON, "catalog update interval (%" PRId64 ") is less than measurements interval (%" PRId64 "). Using the latter.");
+			catalog_interval = interval;
+		}
+
+	} else if (catalog_hosts || catalog_project || catalog_interval) {
+		debug(D_FATAL, "Options --catalog, --catalog-project, and --catalog-interval cannot be used without --catalog-task-name.");
+		exit(RM_MONITOR_ERROR);
+	}
+
+
 	//this is ugly. if -c given, we should not accept any more arguments.
 	// if not given, we should get the arguments that represent the command line.
 	if((optind < argc && sh_cmd_line) || (optind >= argc && !sh_cmd_line && !first_pid_manually_set)) {
@@ -2397,7 +2475,8 @@ int main(int argc, char **argv) {
         debug(D_NOTICE, "using upstream monitor. executing: %s\n", command_line);
         execlp("/bin/sh", "sh", "-c", command_line, (char *) NULL);
         //We get here only if execlp fails.
-        fatal("error executing %s: %s\n", command_line, strerror(errno));
+        debug(D_FATAL, "error executing %s: %s\n", command_line, strerror(errno));
+		exit(RM_MONITOR_ERROR);
     }
 
     write_helper_lib();
