@@ -24,12 +24,14 @@ export CCTOOLS_TEST_TMP=${CCTOOLS_TEST_LOG%.log}.tmp
 
 echo "[$(date)] Testing on $(uname -a)." > "$CCTOOLS_TEST_LOG"
 
-# we need cctools_python in the path.
-PATH="$(pwd)/dttools/src:$PATH"
 # we need resource_monitor in the path.
 PATH="$(pwd)/resource_monitor/src:$PATH"
 export PATH
 
+export CCTOOLS_PYTHON_TEST_EXEC=$(grep "^CCTOOLS_PYTHON_TEST_EXEC=" config.mk | cut -d = -f 2)
+export CCTOOLS_PYTHON_TEST_DIR=$(grep "^CCTOOLS_PYTHON_TEST_DIR=" config.mk | cut -d = -f 2)
+export CCTOOLS_PERL=$(grep "^CCTOOLS_PERL=" config.mk | cut -d = -f 2)
+
 export PYTHONPATH="$(pwd)/chirp/src/python:$(pwd)/work_queue/src/python:$PYTHONPATH"
 export PERL5LIB="$(pwd)/chirp/src/perl:$(pwd)/work_queue/src/perl:$PERL5LIB"
 
