@@ -265,6 +265,13 @@ struct work_queue_stats {
 	int64_t min_memory;       /**< The smallest memory size in MB observed among the connected workers. */
 	int64_t min_disk;         /**< The smallest disk space in MB observed among the connected workers. */
 
+    double master_load;       /**< In the range of [0,1]. If close to 1, then
+                                the master is at full load and spends most
+                                of its time sending and receiving taks, and
+                                thus cannot accept connections from new
+                                workers. If close to 0, the master is spending
+                                most of its time waiting for something to happen. */
+
 	/**< deprecated fields: */
 	int total_workers_connected;    /**< @deprecated Use workers_connected instead. */
 	int total_workers_joined;       /**< @deprecated Use workers_joined instead. */
@@ -964,6 +971,9 @@ void work_queue_master_preferred_connection(struct work_queue *q, const char *pr
  - "fast-abort-multiplier" Set the multiplier of the average task time at which point to abort; if negative or zero fast_abort is deactivated. (default=0)
  - "keepalive-interval" Set the minimum number of seconds to wait before sending new keepalive checks to workers. (default=300)
  - "keepalive-timeout" Set the minimum number of seconds to wait for a keepalive response from worker before marking it as dead. (default=30)
+ - "short-timeout" Set the minimum timeout when sending a brief message to a single worker. (default=5s)
+ - "long-timeout" Set the minimum timeout when sending a brief message to a foreman. (default=1h)
+ - "category-steady-n-tasks" Set the number of tasks considered when computing category buckets.
 @param value The value to set the parameter to.
 @return 0 on succes, -1 on failure.
 */
