@@ -2322,7 +2322,7 @@ static const struct option long_options[] = {
 	{"docker",              required_argument,  0,  LONG_OPT_RUN_DOCKER},
 	{"docker-preserve",     required_argument,  0,  LONG_OPT_RUN_DOCKER_PRESERVE},
 	{"docker-tar",          required_argument,  0,  LONG_OPT_BUILD_FROM_TAR},
-	{"feature",            required_argument,  0,  LONG_OPT_FEATURE},
+	{"feature",             required_argument,  0,  LONG_OPT_FEATURE},
 	{0,0,0,0}
 };
 
@@ -2767,7 +2767,7 @@ int main(int argc, char *argv[])
 
 		if(result) {
 			if(single_shot_mode) {
-				debug(D_NOTICE,"stopping: single shot mode");
+				debug(D_DEBUG,"stopping: single shot mode");
 				break;
 			}
 			backoff_interval = init_backoff_interval;
